@@ -603,7 +603,7 @@ class DictTest(unittest.TestCase):
         # (D) subclass defines __missing__ method returning a value
         # (E) subclass defines __missing__ method raising RuntimeError
         # (F) subclass sets __missing__ instance variable (no effect)
-        # (G) subclass doesn't define __missing__ at all
+        # (G) subclass doesn't define __missing__ at a all
         class D(dict):
             def __missing__(self, key):
                 return 42
@@ -952,92 +952,6 @@ class DictTest(unittest.TestCase):
         d = {X(): 0, 1: 1}
         self.assertRaises(RuntimeError, d.update, other)
 
-    def test_equal_operator_modifying_operand(self):
-        # test fix for seg fault reported in issue 27945 part 3.
-        class X():
-            def __del__(self):
-                dict_b.clear()
-
-            def __eq__(self, other):
-                dict_a.clear()
-                return True
-
-            def __hash__(self):
-                return 13
-
-        dict_a = {X(): 0}
-        dict_b = {X(): X()}
-        self.assertTrue(dict_a == dict_b)
-
-    def test_fromkeys_operator_modifying_dict_operand(self):
-        # test fix for seg fault reported in issue 27945 part 4a.
-        class X(int):
-            def __hash__(self):
-                return 13
-
-            def __eq__(self, other):
-                if len(d) > 1:
-                    d.clear()
-                return False
-
-        d = {}  # this is required to exist so that d can be constructed!
-        d = {X(1): 1, X(2): 2}
-        try:
-            dict.fromkeys(d)  # shouldn't crash
-        except RuntimeError:  # implementation defined
-            pass
-
-    def test_fromkeys_operator_modifying_set_operand(self):
-        # test fix for seg fault reported in issue 27945 part 4b.
-        class X(int):
-            def __hash__(self):
-                return 13
-
-            def __eq__(self, other):
-                if len(d) > 1:
-                    d.clear()
-                return False
-
-        d = {}  # this is required to exist so that d can be constructed!
-        d = {X(1), X(2)}
-        try:
-            dict.fromkeys(d)  # shouldn't crash
-        except RuntimeError:  # implementation defined
-            pass
-
-    def test_dictitems_contains_use_after_free(self):
-        class X:
-            def __eq__(self, other):
-                d.clear()
-                return NotImplemented
-
-        d = {0: set()}
-        (0, X()) in d.items()
-
-    def test_init_use_after_free(self):
-        class X:
-            def __hash__(self):
-                pair[:] = []
-                return 13
-
-        pair = [X(), 123]
-        dict([pair])
-
-    def test_oob_indexing_dictiter_iternextitem(self):
-        class X(int):
-            def __del__(self):
-                d.clear()
-
-        d = {i: X(i) for i in range(8)}
-
-        def iter_and_mutate():
-            for result in d.items():
-                if result[0] == 2:
-                    d[2] = None # free d[2] --> X(2).__del__ was called
-
-        self.assertRaises(RuntimeError, iter_and_mutate)
-
-
 from test import mapping_tests
 
 class GeneralMappingTests(mapping_tests.BasicTestMappingProtocol):
@@ -1049,12 +963,5 @@ class Dict(dict):
 class SubclassMappingTests(mapping_tests.BasicTestMappingProtocol):
     type2test = Dict
 
-def test_main():
-    support.run_unittest(
-        DictTest,
-        GeneralMappingTests,
-        SubclassMappingTests,
-    )
-
 if __name__ == "__main__":
-    test_main()
+    unittest.main()
